
# babackup

Babackup is a tool providing secure, time-traveling backup via ssh and rsync.
It has the following goals:

- No mysteries: It uses standard tools (rsync, ssh), 
    and storage formats (the file system)
    so it's easy to inspect what's going on,
    verify it's happy, and recover from the unexpected.
    
- High security:  Remote access is limited, so compromise of either
    a client or the server have a limited blast radius.
    
    Compromise of a client can at most cost the current and
    immediately prior backup, but cannot spread to remote access of
    the server or the entire backup history.
    
    Compromise of the server cannot spread to remote access of
    clients.  However, the server has a clear-text backup of the data
    from the client, so the data may be disclosed.

    Both the client and server sides can run with or without
    privilege.  With privilege (as root), file user and permissions
    are preserved.
    
    For these reasons, the both the client and the server can be 
    considered "high security" machines, but compromise of either
    will not spread to the other.
    
- History:  One can walk back history, time-machine like.  History is
    adaptively aged so we remember a lot about the near past and some
    about the far past.

- Storage efficiency:  Backups are de-duplicated, so if nothing
    changes from day to day, the incremental storage cost is minimal.
    
We expand on how we accomplish these goals in
["Technical Details"](#Technical Details) below.


# Using Babackup

## Normal Operation

On the client, run `babackup` periodically
(say, from cron).  That will see when backups last run, and run a new
one if it's been more than a day.  This side pushes a new, incremental
backup to the server.

Babackup reads its default client configuration from 
`~/.config/babackup/client.yaml` or `/etc/babackup/client.yaml`
(if running as non-root or root).  Other configurations can be
specified with `-c`.

On the server, run `babackup_server` 
frequently (say, every 5 or 10 minutes).  It will detect when a
new backup has completed and prepare for the next one.
It will also periodically (daily) look over old backups and age them
out.  (Server-side configurations are in `server.yaml`.


## New Backups

### New Backups: On the Client

To start backing up a new partition,
run `babackup -n /new/backup/path -s serveruser@serverhost:server-side/storage/location` 
on the client. 
This command will add the new backup path to the
client configuration, and print out a command (described below) 
to run on the server-side. 

(Optionally, add `-N name` to give this backup a "name".
If you don't specify a name,
a numeric one is generated off the client-side path.)

Client-side setup updates the configuration
files so future runs know what to do.
It also generates a new, passwordless ssh key
to use for this backup.

### New Backups: On the Server

To start backups on the server side for the first time,
intsall the software and ensure `babackup_server` will run frequently
(more frequently than backups occur, so at least every 30 minutes),
as whatever user is receving the data.

When a new client-side backup is started,
cut-and-paste what it tells you to run on the server side.

On the server side, that command installs the ssh key with an rsync
limitation to the backup directory,
and updates server-side configuration files so it knows
to watch the new backup,
and it initializes the storage location.


# Technical Details

Some technical details about how we meet our goals.

## Security

For security, all client-to-server remote access is done over ssh, 
protecting from eavesdropping and limiting remote access.

The server-side uses rrsync to limit remote access to writing to
the backup directory.  Therefore the client has no remote shell access
to the server, and the server has no access to the client.

This model means the client can use passwordless-ssh keys without
concern that an attacker can gain lateral movement between hosts.

Data restoration (that is: server-to-client data movement) is handled
out-of-band.  The client and the server users may be different people,
and we assume they can communicate out of band to do data recovery.

(Alternatively, a second ssh key could allow client-to-server
read-only access to data archive, to allow data recovery without
intervention from the server operator.)

## Data storage: simplicity, history, and efficiency

For simplicity and history of data storage, backups just go to the
file system.  Each day is in a separate date-stamped directory tree
(like Plan 9 backups, or Apple Time Machine).

For history, we age out old backups.  Typically we keep the N most
recent days (default: N = 10), and one backup per week for the M weeks
beyond "recent" (default: M = 10), allowing one to look at any day
from the last week and a half, and any week from the last 2.5 months.

For efficiency, backups use hardlinks for unchanged files, so they are
de-duplicated (an unchanged file takes nearly no extra space from day
to day).  (Hard links are managed by the rsync --link-dest option.)

## Alternatives

In a slightly better world we would encrypt backups before leaving the
client, allowing the server to be untrusted.  This approach was not
chosen because it lot more client-side infrastructure, making it
impossible to use unmodified rsync.


# History

## Release History

- 1.0 2024-01-14 First public release.

- 1.1 2024-01-15 Improve handling of --new-mode={rrsync,ssh}

## Ancient History

A first version of these ideas (rsync and a link-dest tree, inspired
by a post on the Internet pointing out the power of link-dest) was
deployed in 1999 by John Heidemann to back up his laptop.  This
version was mostly shell scripts (with more shell scripts for
configuration files), but most configuration was done by hand.
(Inspired by Ken Thompson, "the experienced user will usually know
what to configure".)

Around 2015 Yuri adapted these scripts for lab use.

In 2024 I rewrote the shell scripts in python to take over
configuration, and to enable an rrsync-protected mode.
(In response to an increased security threat of lateral movement via
passwordless ssh keys, the need to backup a semi-trusted laptop,
and the need to have high security clients and backup servers
that are still outside the "compromise" blast radius.

